unit DCOM_MDAC_Versions;

{ -----------------------------------------------------------------------------}
{ Description: DCOM_MDAC_Versions v1.00                                        }
{ Author:      Kevin S. Gallagher                                              }
{ Copyrights:  Project JEDI                                                    }
{ -----------------------------------------------------------------------------}
{ Functions for determining the installed versions of Microsoft DCOM and MDAC. }
{ -----------------------------------------------------------------------------}
{ Tested under Windows95 and Windows98                                         }
{ -----------------------------------------------------------------------------}
{ Date last modified:  Feburary 17, 2000                                       }
{ -----------------------------------------------------------------------------}


interface

uses Windows, Registry, SysUtils, ActiveX, ComObj ;

type
  TCoCreateInstanceExProc = function (const clsid: TCLSID;
    unkOuter: IUnknown; dwClsCtx: Longint; ServerInfo: PCoServerInfo;
    dwCount: Longint; rgmqResults: PMultiQIArray): HResult stdcall ;


function GetMDACVersion: String ;
function DCOMEnabled: boolean ;
function GetDCOMVersion: String ;

implementation

function RemoveCommas(const cMsg:String):String ;
var nLen, i:Integer ;
begin
  nLen := Length(cMsg) ;
  i := 1 ;
  while i <= nLen do begin
    if cMsg[i] in [','] then
      Result := Result + '.'
    else
      Result := Result + cMsg[i];
    Inc(i) ;
  end ;
end ;

function GetVerNoString(FileName: string): string ;
type
  TTRANSARRAY = packed record
    LangID: WORD;
    CharSet: WORD;
  end;
  PTRANSARRAY = ^TTRANSARRAY ;
var
  dwSize, dwZero: DWORD ;
  Buff, pVer: Pointer ;
  pTA: PTRANSARRAY ;
  LangCharSet: string ;
begin
  Result := '' ;
  
  dwSize := GetFileVersionInfoSize(PChar(FileName), dwZero) ;
  GetMem(Buff, dwSize) ;
  try
    if GetFileVersionInfo(PChar(FileName), dwZero, dwSize, Buff) then
    begin
      VerQueryValue(Buff,PChar('\VarFileInfo\Translation'),Pointer(pTA),dwSize) ;
      with pTA^ do
       LangCharSet := Format('\StringFileInfo\%s%s\FileVersion',
         [IntToHex(LangID, 4), IntToHex(charset, 4)]) ;
      VerQueryValue(Buff, PChar(LangCharSet), pVer, dwSize) ;
      Result := StrPas(PChar(pVer)) ;
    end ;
  finally
   FreeMem(Buff) ;
  end ;
end ;

{***************************************************************************}
{ Determines if MS-DCOM is enabled.                                         }
{***************************************************************************}
function DCOMEnabled: boolean ;
const
  SDCOMNotInstalled = 'DCOM not enabled' ;
var
  Ole32: HModule ;
  CoCreateInstanceEx: TCoCreateInstanceExProc ;
  OldError: Longint ;
begin
  Result := False ;

  CoCreateInstanceEx := nil ;

  OldError := SetErrorMode(SEM_NOOPENFILEERRORBOX) ;

  try
    if @CoCreateInstanceEx = nil then
    begin
      Ole32 := GetModuleHandle('ole32.dll') ;
      Win32Check(Ole32 > HINSTANCE_ERROR) ;
      @CoCreateInstanceEx := GetProcAddress(Ole32, 'CoCreateInstanceEx') ;
      if @CoCreateInstanceEx = nil then
        Result := False
      else
        Result := True ;
    end ;
  finally
    SetErrorMode(OldError) ;
  end ;

end ;

{***************************************************************************}
{ Returns the installed version of DCOM                                     }
{ The version information is stored with commas, if you rather return it    }
{ with decimals then toggle the commented result below.                     }
{***************************************************************************}
function GetDCOMVersion: String ;
const
  sSubKey = 'CLSID\{bdc67890-4fc0-11d0-a805-00aa006d2ea4}\InstalledVersion' ;
var
  Reg: TRegistry ;
begin

  if not DCOMEnabled then
  begin
    Result := '' ;
    { No need to continue. Of course we could
      have coded this a bit better, but this
      is clearer. }
    exit ;
  end ;

  Reg := TRegistry.Create ;

  try
    Reg.RootKey := HKEY_CLASSES_ROOT ;
    try
      if Reg.OpenKey(sSubKey,False) then
        if Reg.ValueExists('') then
          // Result := RemoveCommas(Reg.ReadString('')) ;
          Result := Reg.ReadString('') ;
    except
      Result := 'ERROR' ;
    end ;
  finally
    Reg.Free ;
  end ;

end ;

{***************************************************************************}
{ Returns the version of MDAC                                               }
{***************************************************************************}
function GetMDACVersion: String ;
var
  Reg: TRegistry ;
  cKey: String ;
  cDll: String ;
begin

  { Default return value, You might want to simply raise an exception. }
  Result := '' ;

  Reg := TRegistry.Create ;

  try
    Reg.RootKey := HKEY_CLASSES_ROOT ;
    if Reg.OpenKey('ADODB.Connection\CLSID',False) then
      if Reg.ValueExists('') then
      begin
        cKey := Reg.ReadString('') ;
        Reg.CloseKey ;
        if Reg.OpenKey('CLSID\' + cKey + '\InprocServer32' ,False) then
          if Reg.ValueExists('') then
          begin
            cDll := Reg.ReadString('') ;
            Reg.CloseKey ;
            Result := GetVerNoString(cDll) ;
          end ;
      end ;
  finally
    Reg.Free ;
  end ;

end ;

end.
