unit Insight;

{$I JCL.INC}

{*********************************************************************}
{ Description:  An excercise to examine various things of Delphi.     }
{  Programmer:  Kevin S Gallagher     gallaghe@teleport.com           }
{    Versions:  3                                                     }
{ Suggestions:  Think about adding for working with packages,         }
{               version information also.                             }
{                                                                     }
{ The constant KG_DUMMYDIR is used in "Save File" event, make sure to }
{ read my comments before undefining it.                              }
{                                                                     }
{ Please note, I could have done a RichText component for the code    }
{ and done highlighten like in Delphi but do not have the time.       }
{*********************************************************************}

{$DEFINE KG_DUMMYDIR}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Grids,FileCtrl;

type
  TfrmInsight = class(TForm)
    pnlMenu: TPanel;
    cmdSaveChanges: TButton;
    cmdExit: TButton;
    StatusBar1: TStatusBar;
    pnlCode: TPanel;
    lblShortCut: TLabel;
    lblDescription: TLabel;
    txtShortCut: TEdit;
    txtDesc: TEdit;
    TheCode: TMemo;
    sGrid: TStringGrid;
    Bevel1: TBevel;
    cmdApplyChanges: TButton;
    cmdAddNewShortCut: TButton;
    cmdDeleteShortCut: TButton;
    procedure cmdExitClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure sGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure sGridKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure sGridClick(Sender: TObject);
    procedure sGridEnter(Sender: TObject);
    procedure sGridDblClick(Sender: TObject);
    procedure cmdSaveChangesClick(Sender: TObject);
    procedure cmdApplyChangesClick(Sender: TObject);
    procedure cmdAddNewShortCutClick(Sender: TObject);
    procedure cmdDeleteShortCutClick(Sender: TObject);
  private
    function GetShortCut:String;
    function GetDesc:String;
    function GetMemoCode:String;
    procedure ApplyNewValues;
    procedure RefreshItems;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmInsight: TfrmInsight;
  aLines:TStringList;

implementation

uses DPRegstr, JEDI_Dialogs;

{$R *.DFM}

procedure TfrmInsight.cmdExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmInsight.FormActivate(Sender: TObject);
var
  i,x:Integer;
  cTemp:String ;
begin

  sGrid.Cells[0,0] := 'Name/ShortCut';
  sGrid.Cells[1,0] := 'Description';

  try
    ReadCodeInSight;

    x := 1;

    for i := 0 to MAX_CODE -1 do
      if length(aString[i,0]) > 0 then begin
        Inc(x);
        sGrid.Cells[0,i +1] := aString[i,0];
        sGrid.Cells[1,i +1] := aString[i,1];
        aLines.Add(aString[i,2]);
        aString[i,0] := '';
        aString[i,1] := '';
        aString[i,2] := '';
      end;
    sGrid.RowCount := x;

    RefreshItems;

  except  // need to finish up
    ShowMessage(SysErrorMessage(  GetLastError  )) ;
    Application.ProcessMessages ;
    Close ;
  end;
end;

procedure TfrmInsight.sGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  RefreshItems;
end;

procedure TfrmInsight.sGridKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  RefreshItems;
end;

function TfrmInsight.GetMemoCode:String;
begin
  Result := aLines.Strings[sGrid.Row -1]
end;

function TfrmInsight.GetShortCut:String;
begin
  Result := sGrid.Rows[sGrid.Row].Strings[0];
end;

procedure TfrmInsight.ApplyNewValues;
var
  S:String;
  i:Integer;
begin
  S := '';
  sGrid.Rows[sGrid.Row].Strings[0] := txtShortCut.text;
  sGrid.Rows[sGrid.Row].Strings[1] := txtDesc.text;
  for i := 0 to TheCode.Lines.Count -1 do
    if i = TheCode.Lines.Count -1 then
      S := S + TheCode.Lines[i]
    else
      S := S + TheCode.Lines[i] +#13#10;

  aLines.Strings[sGrid.Row -1] := S;
end;

function TfrmInsight.GetDesc:String;
begin
  Result := sGrid.Rows[sGrid.Row].Strings[1];
end;

procedure TfrmInsight.RefreshItems;
begin
  try
    TheCode.Lines.Clear;
    TheCode.Lines.Add(GetMemoCode);
    TheCode.SelStart := 0;
    TheCode.SelLength := 0;

    txtShortCut.text := GetShortCut;
    txtDesc.text := GetDesc;
  except
  end;
end;

procedure TfrmInsight.sGridClick(Sender: TObject);
begin
  RefreshItems;
end;

procedure TfrmInsight.sGridEnter(Sender: TObject);
begin
  RefreshItems;
end;

procedure TfrmInsight.sGridDblClick(Sender: TObject);
begin
  RefreshItems;
end;

{*******************************************************************
  Purpose:
  To save changes back to disk. I made a decision not to save
  any changes back to the original file, but instead write the
  changes to the project directory with a filename of [Delphi32.ksg]
  You must alter the code to write back to [Delphi32.dci], the file
  which Delphi reads from. This insures that you can not blame me
  for anything bad occuring!
*******************************************************************}
procedure TfrmInsight.cmdSaveChangesClick(Sender: TObject);
var
  F:Textfile;
  cDir:String;
  cFile:String;
  x:Integer;
  iLen:Integer;
begin

  {$IFDEF KG_DUMMYDIR}
    cFile :=  'Delphi32.ksg';
    cDir := ExtractFilePath(Application.ExeName);
  {$ELSE}
    cFile :=  'Delphi32.dci';
    cDir := WhereIsDelphi + #92  + 'Bin\';
  {$ENDIF}

  if not DirectoryExists(cDir) then
    ShowMessage('Could not locate ' + cDir);

  try
    AssignFile(F,cDir + #92 + cFile);
    ReWrite(F);
    {$IFDEF KG_DUMMYDIR}
    WriteLn(F,'Before undefining KG_DUMMYDIR please inspect all lines');
    WriteLn(F,'to insure they confirm with structure Delphi uses.');
    WriteLn(F,'');
    {$ENDIF}
    for x := 1 to sGrid.RowCount -1 do begin
      Writeln(F,
        '[' + trim(sGrid.Rows[x].Strings[0]) + #32 + '| ' +
              trim(sGrid.Rows[x].Strings[1]) +       ']'
      );

      iLen := Length(aLines.Strings[x -1]);

      if iLen >0 then
        Writeln(F,aLines.Strings[x -1]);

      WriteLn(F,#32);
    end;
  finally
    Closefile(F);
  end;
end;

procedure TfrmInsight.cmdApplyChangesClick(Sender: TObject);
begin
  ApplyNewValues;
end;

procedure TfrmInsight.cmdAddNewShortCutClick(Sender: TObject);
var
  n:word;
begin
  if not Question('Add new item?') then
    exit;

  with sGrid do begin
    Row := RowCount -1;
    RowCount := RowCount +1;
    Row := RowCount -1;
    Rows[Row].Strings[0] := 'Enter a shortcut';
    Rows[Row].Strings[1] := 'Enter a Description';
    aLines.Add('Enter your code here!!!');
  end;
  RefreshItems;
end;

procedure TfrmInsight.cmdDeleteShortCutClick(Sender: TObject);
var
  n:word;
begin
  with sGrid do begin
    if RowCount -1 = FixedRows then
      exit;

  if not Question('Really delete this item') then
    exit;

    aLines.Delete(sGrid.Row -1);

    for n := Row to RowCount -2 do
      Rows[n] := Rows[n +1];

    RowCount := RowCount -1;
    Rows[RowCount].Clear;
    SetFocus;
  end;
  RefreshItems;
end;

initialization
  aLines:= TStringList.Create;
  aLines.Duplicates := dupIgnore;
finalization
  aLines.Free;

end.
