unit JEDI_Dialogs;

{$I JCL.INC}

{*******************************************************************}
{ Programmer:   Kevin S. Gallagher                                  }
{ Description:  Various wrappers for showing information in dialogs }
{                                                                   }
{ These routines were created for a JEDI article                    }
{ http://www.delphi-jedi.org/Jedi:VOYFORMS4:875264673               }
{*******************************************************************}

interface

uses  Windows, Messages, SysUtils, Controls, Forms, Dialogs, StdCtrls;


function DefMessageDlg(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       DefButton: Integer;
                       HelpCtx: Longint): Integer;


{$IFDEF DELPHI_5}
function MessageDlgCtr(const Msg: string; DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       bCenterOnForm: boolean = True): Integer; overload ;

function MessageDlgCtr(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       HelpCtx: Longint;
                       bCenterOnForm: boolean = True): Integer; overload ;


{$ELSE}
function MessageDlgCtr(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       HelpCtx: Longint;
                       bCenterOnForm: boolean): Integer ;

{$ENDIF}


function Question(cQuestion: String):boolean ;
procedure ErrorMsg(cMessage:String) ;
procedure MsgBox(cMessage:String) ;

implementation

function StrCrLf(const cMsg:String):String;
var nLen,i:Integer;
begin

  nLen := Length(cMsg);
  i := 1;
  while i <= nLen do begin
    if cMsg[i] in [';','~'] then
      Result := Result + #13
    else
      Result := Result + cMsg[i];
    Inc(i);
  end;
end;


function Question(cQuestion: String):boolean ;
begin
  Result := (MessageDlgCtr('Question',cQuestion,mtInformation,[mbYes,mbNo],0,True) = mrYes) ;
end ;

procedure ErrorMsg(cMessage:String) ;
begin
  MessageDlgCtr('ERROR',cMessage,mtError,[mbOk],0,True)
end ;

procedure MsgBox(cMessage:String) ;
begin
  MessageDlgCtr('',cMessage,mtInformation,[mbOk],0,True)
end ;

function DefMessageDlg(const aCaption: string; const Msg: string;
  DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; DefButton: Integer;
  HelpCtx: Longint): Integer;
var
  i: Integer;
  btn: TButton;
begin
  with CreateMessageDialog(StrCrLf(Msg), DlgType, Buttons) do
  try
    Caption := aCaption ;
    HelpContext := HelpCtx ;
    for i := 0 to ComponentCount -1 do begin
      if Components[i] Is TButton then begin
        btn := TButton(Components[i]) ;
        btn.default:= btn.ModalResult = DefButton ;
        if btn.default then
          ActiveControl := Btn ;
      end;
    end ;
    Result := ShowModal ;
  finally
    Free ;
  end ;
end;


{$IFDEF DELPHI_5}
function MessageDlgCtr(const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
                       bCenterOnForm: boolean = True): Integer; overload ;
begin
  bCenterOnForm := Screen.ActiveForm <> nil ;
  with CreateMessageDialog(StrCrLf(Msg), DlgType, Buttons) do
    try
      if bCenterOnForm then begin
        Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2);
        Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2);
      end ;
      Result := ShowModal;
    finally
      Free;
    end;
end;

function MessageDlgCtr(const aCaption: string; const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
  HelpCtx: Longint; bCenterOnForm: boolean = True): Integer;
begin
  bCenterOnForm := Screen.ActiveForm <> nil ;
  with CreateMessageDialog(StrCrLf(Msg), DlgType, Buttons) do
    try
      Caption := aCaption ;
      HelpContext := HelpCtx;
      if bCenterOnForm then begin
        try
          Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2);
          Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2);
        except
        end ;
      end ;
      Result := ShowModal;
    finally
      Free;
    end;
end;
{$ELSE}
function MessageDlgCtr(const aCaption: string; const Msg: string;
                       DlgType: TMsgDlgType;
                       Buttons: TMsgDlgButtons;
                       HelpCtx: Longint;
                       bCenterOnForm: boolean): Integer ;
begin
  bCenterOnForm := Screen.ActiveForm <> nil ;
  with CreateMessageDialog(StrCrLf(Msg), DlgType, Buttons) do
    try
      Caption := aCaption ;
      HelpContext := HelpCtx;
      if bCenterOnForm then begin
        try
          Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) -(Width div 2);
          Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) -(Height div 2);
        except
        end ;
      end ;
      Result := ShowModal;
    finally
      Free;
    end;
end;
{$ENDIF}
end.


