unit MainForm;
{***************************************************************************}
{ Author                                                                    }
{   Kevin S. Gallagher                                                      }
{   gallaghe@teleport.com                                                   }
{ Description                                                               }
{   This project is for shown how to work with the registry in Delphi for   }
{   JEDI Education.                                                         }
{                                                                           }
{   Prior to venturing out on your own with the registry, make sure you DO  }
{   understand what can happen if you mess something up. One of the goals   }
{   for this lesson is to assist you in causing any problems.               }
{ Comments                                                                  }
{   + No third party components used or are needed to compile the project.  }
{   + Every now and then will testing the code I got some violations from   }
{     Windows. Nothing that was repeatable, nothing damaging either.        }
{   + The code has been highly tested, nothing is taken for granted when it }
{     comes to causing any problems to your computer.                       }
{   + Since I do not have NT there may be security issues which I have not  }
{     or could not address. Bottom line, you may not be able to write       }
{     information back without doing more coding. If I can get someone who  }
{     has NT and knows how to work the registry under NT can provide me     }
{     with feedback, then I will update this project to adhere to Windows NT}
{     security. I should be getting NT soon so I hope to be able to update  }
{     this project as I learn more about NT security.                       }
{   + The keys used in OnCreate and OnClose are not removed in this demo.   }
{     you need to remove them, either with RegEdit or creating some code to }
{     handle the removal of the keys. If you need assistance let me know.   }
{***************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Menus, ExtCtrls, StdCtrls;

type
  TMyFormSettings = Record
    Top:Integer ;
    Left:Integer ;
    Width:Integer ;
    Height:Integer ;
    WindowState:Integer ;
  end ;
  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    mnuExit: TMenuItem;
    mnuDemos: TMenuItem;
    StatusBar1: TStatusBar;
    Label1: TLabel;
    Bevel1: TBevel;
    Arewesurfing1: TMenuItem;
    mnuGetADO_Info: TMenuItem;
    mnuGetUserColors: TMenuItem;
    mnuGetonecolor: TMenuItem;
    mnuReadWritecurrencyandbinarydata1: TMenuItem;
    ListBox1: TListBox;
    mnuReadDelphisCodeInsight: TMenuItem;
    mnuGetDelphisessions: TMenuItem;
    Help1: TMenuItem;
    mnuAbout: TMenuItem;
    mnuMSWordinformation: TMenuItem;
    ChangeNovelldefaultuser1: TMenuItem;
    mnuRunonWindowsstartup: TMenuItem;
    mnuBrowsebranchoftheregistry: TMenuItem;
    mnuRestoreTreeView: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Arewesurfing1Click(Sender: TObject);
    procedure mnuGetADO_InfoClick(Sender: TObject);
    procedure mnuGetUserColorsClick(Sender: TObject);
    procedure mnuGetonecolorClick(Sender: TObject);
    procedure mnuReadWritecurrencyandbinarydata1Click(Sender: TObject);
    procedure mnuReadDelphisCodeInsightClick(Sender: TObject);
    procedure mnuGetDelphisessionsClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuMSWordinformationClick(Sender: TObject);
    procedure ChangeNovelldefaultuser1Click(Sender: TObject);
    procedure mnuRunonWindowsstartupClick(Sender: TObject);
    procedure mnuBrowsebranchoftheregistryClick(Sender: TObject);
    procedure mnuRestoreTreeViewClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    cNovellError:String ;
    procedure AppOnHint(Sender: TObject);
  end;

procedure CenterForm(f : TForm) ;

const
  { Both of these constants are used to access the registry
    in this form's OnCreate and OnClose events }
  MyBaseKey       = 'JEDI_Registry_Demo' ;
  MyAppSettings   = 'FormsSettings' ;
  MyTreeNodeKey   = 'RestoreTree' ;
  
var
  frmMain: TfrmMain;

implementation

uses registry, IE_Connected, JEDI_Dialogs, DCOM_MDAC_Versions,
  ShowCurrentUserColors, Insight, DPRegstr, ShellApi, MS_Word_Info,
  Novell_LogOn, RunEntries, Unit2, RestoreTreeNode ;

{$R *.DFM}


{--------------------------------------------------------
  Used to center a child form over the calling form.
  It assumes that someone called it i.e. another form.
  If no active form an execption is raised! If you want
  to protect the code, refer to code in JEDI_Dialogs for
  code to do so.
 -------------------------------------------------------}
procedure CenterForm(f : TForm) ;
begin
  f.Left := Screen.ActiveForm.Left + (Screen.ActiveForm.Width div 2) - (f.Width div 2) ;
  f.Top := Screen.ActiveForm.Top + (Screen.ActiveForm.Height div 2) - (f.Height div 2) ;
end ;


{--------------------------------------------------------
  The code here works along with OnClose code to restore
  this forms position to what it was when last closed.
 -------------------------------------------------------}
procedure TfrmMain.FormCreate(Sender: TObject);
var
  { This variable is used when an integer is read from the registry
    which will be used to compare the prior state of this form. We
    typecast the integer to TWindowState for comparing the last known
    state to the current Window State.}
  PriorState:TWindowState ;

  { This variable is used to access the registry key we are to read
    the forms position from. }
  Reg: TRegistry ;
begin
  cNovellError := 'Unknown error' ;
  { Create the object }
  Reg := TRegistry.Create ;

  try
    { 1.  Usually prior to opening a key in the registry you
          must set the the "base" key, known as the "rootkey"
          unless it happens to be HKEY_CURRENT_USER, which it
          is in this case. So there is no need to set a "root
          key".

      2.  Setting the second parameter to False tells our
          object to only open the key if it exist. If it does
          not then the code below it does not get executed.
          If you want to "force" the key to open if it does not
          exist then use <True> as the second parameter. There
          is no need to do this here since we only write to the
          key in the OnClose event of this form.
      }
    if Reg.OpenKey(MyBaseKey,False) then
    begin
      if Reg.OpenKey(MyAppSettings,True) then
      begin
        with Reg do
        begin
          { For the code to function properly we need all five
            values stored in the registry. Hence a check is
            needed to insure that these values exist prior to
            attempting to use them.}
          if ValueExists('Top')   and ValueExists('Left')   and
             ValueExists('Width') and ValueExists('Height') and
             ValueExists('WindowState') then
          begin
            { If you got here then all five values exist, using
              SetBounds is better then setting all four properties
              one at a time. }
            Self.SetBounds(ReadInteger('Left'),
                           ReadInteger('Top'),
                           ReadInteger('Width'),
                           ReadInteger('Height')) ;

            { Typecase the integer returned from the registry so
              we can compare the prior "WindowState" to the
              current WindowState.  }
            PriorState := TWindowState(ReadInteger('WindowState')) ;

            if PriorState = wsMinimized then
            begin
              Visible := True ;
              Application.Minimize  ;
            end else
              WindowState := PriorState ;
          end ;
        end ;
      end ;
    end ;
  finally
    { Clean up after ourself. }
    Reg.Free ;
  end ;
end;

{--------------------------------------------------------
  The code here works along with OnCreate code to save
  this forms position so it can be restored next time it
  is executed.
 -------------------------------------------------------}
procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Reg: TRegistry ;
begin
  Reg := TRegistry.Create ;
  try
    { Here we need the key(s) to exist if it does not currently
      exist, so we pass <True> as the second parameter. }
    if Reg.OpenKey(MyBaseKey,True) then
      if Reg.OpenKey(MyAppSettings,True) then
      begin
        with Reg do
        begin
          { Store the current form settings }
          WriteInteger('Top',Top) ;
          WriteInteger('Left',Left) ;
          WriteInteger('Width',Width) ;
          WriteInteger('Height',Height) ;
          WriteInteger('WindowState',Ord(WindowState)) ;
          CloseKey ;
        end ;
      end ;
  finally
    Reg.Free ;
  end ;
end;

{--------------------------------------------------------
  This code insures that hints are shown on the current
  form. If the current form has no component named
  StatusBar1 then no errors occur. It assumes an active
  form.
 -------------------------------------------------------}
procedure TfrmMain.AppOnHint ;
var
  T: TComponent ;
begin
  T := Screen.ActiveForm.FindComponent('Statusbar1') ;
  if T <> nil then
    TStatusBar(T).SimpleText := Application.Hint ;
end ;

procedure TfrmMain.mnuExitClick(Sender: TObject);
begin
  Close ;
end;

procedure TfrmMain.FormResize(Sender: TObject);
begin
  { Some crude resizing code }
  Label1.Width := ClientWidth - 12 ;
  Bevel1.Width := ClientWidth - 8 ;

  with ListBox1 do
  begin
    Top := (Bevel1.Top + Bevel1.Height) + 5 ;
    left := Bevel1.Left ;
    Width := Bevel1.Width ;
    Height := 90 ;
  end ;
end;

{--------------------------------------------------------
  Uses registry Windows API to determine if there is an
  active internet connection. This only works for local
  modem connections. If using a LAN connection it most
  likely will not work.
 -------------------------------------------------------}
procedure TfrmMain.Arewesurfing1Click(Sender: TObject);
begin
  if IsConnected then
    MsgBox('Connected')
  else
    MsgBox('Not connected') ;
end;

{--------------------------------------------------------
  Demos how to read several registry keys to determine the
  version of MS-DCOM and MS-MDAC.

  Side note: There are different installations for both
  DCOM and MDAC i.e. one for Windows95, one for Win98.
  Both Windows98 and Windows2000 come with DCOM and MDAC
  installed. They may not be up todate versions.
  Microsoft has a page which will tell you the current
  versions. For example, at this time 2.1 something
  is current for anything other then Windows2000 which
  is version 2.5 at this time.The 2.5 version adds some
  hyperlink options to ADO (as heard at VBITS-2000).
 -------------------------------------------------------}
procedure TfrmMain.mnuGetADO_InfoClick(Sender: TObject);
begin
  if DCOMEnabled then
  begin
    MsgBox('ADO/MDAC version = ' + GetMDACVersion + #13 + 'DCOM version = ' +
                               GetDCOMVersion );
  end else ErrorMsg('DCOM not enabled') ;
end;

{--------------------------------------------------------
  Shows how to get several values from a branch in the
  registry and convert them to something we can view.
 -------------------------------------------------------}
procedure TfrmMain.mnuGetUserColorsClick(Sender: TObject);
var
  f: TfrmShowUserColors ;
begin
  f := TfrmShowUserColors.Create(Self) ;
  try
    f.Constraints.MaxHeight := ClientHeight - 10 ;
    f.Constraints.MaxWidth := ClientWidth - 10 ;
    CenterForm(f) ;
    f.ShowModal ;
  finally
    f.free ;
  end ;
end;

{--------------------------------------------------------
  Shows how to read a single value from the registry
  which is not formatted as in the above code.
 -------------------------------------------------------}
procedure TfrmMain.mnuGetonecolorClick(Sender: TObject);
const
  BaseKey = 'Control Panel\Colors' ;
  TheKey  = 'ButtonFace' ;
var
  Reg: TRegistry ;
begin
  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      //This is the default --> RootKey := HKEY_CURRENT_USER ;
      if OpenKey(BaseKey, False) then
        if ValueExists(TheKey) then
          ShowMessage('ButtonFace= ' + ReadString('ButtonFace')) ;
    end ;
  finally
    Reg.Free ;
  end ;
end;

{--------------------------------------------------------
  Shows how to read and write different data types from
  the registry including binary and currency types.
 -------------------------------------------------------}
procedure TfrmMain.mnuReadWritecurrencyandbinarydata1Click(
  Sender: TObject);
{ This record is used to demo read/write operations to
  the registry }
type
  TSomeData = record
     Top,
     Left,
     Width,
     Height:Integer ;
     Caption:String ;
   end;

{ Key beneath HKEY_CURRENT_USER to work with }
const
  BaseKey = 'a_test' ;

var
  Reg: TRegistry ;
  MyCurrency: Currency ;
  FSomeData : TSomeData ;
begin
  if not ListBox1.Visible then
    ListBox1.Visible := True ;
    
  ListBox1.Clear ;

  { Assign a values to store into the registry }
  FSomeData.Top := Top ;
  FSomeData.Left := Left ;
  FSomeData.Width := Width ;
  FSomeData.Height := Height ;
  FSomeData.Caption := Caption ;

  MyCurrency := 1000000.90 ;

  { Create an object to access the system registry }
  Reg := TRegistry.Create ;

  try
    Reg.RootKey := HKEY_CURRENT_USER ;
    if not Reg.CreateKey(BaseKey) then
    begin
      ShowMessage('Failed to make it new entry for "' + BaseKey + '"');
      exit ;
    end ;
    Reg.OpenKey(BaseKey,False) ;

    { Write our data to the registry }
    Reg.WriteCurrency('Currency Example',MyCurrency) ;
    Reg.WriteBinaryData('BinaryData Example',FSomeData,SizeOf(TSomeData)) ;


    { Now read back the data for currency }
    ListBox1.Items.Add(Format('Currency Example = %m',
      [Reg.ReadCurrency('Currency Example')])) ;
    ListBox1.Refresh ;

    { Nullify the record to insure we get back the data }
    FSomeData.Top := -1 ;
    FSomeData.Left := -1 ;
    FSomeData.Width := -1 ;
    FSomeData.Height := -1 ;
    FSomeData.Caption := '' ;

    { Screw a bit with the form, later we restore it with the
      registry data }
    Top := Top - 100 ;
    Left := Left - 100 ;
    Caption := 'Just demoing...' ;

    { Timely pause for reflection }
    Sleep(2000) ;

    { Okay let's get the data back }
    Reg.ReadBinaryData('BinaryData Example',FSomeData,SizeOf(TSomeData)) ;

    { Show the returned members retrieved from the registery }
    ListBox1.Items.Add('Top = ' + IntToStr(FSomeData.Top)) ;
    ListBox1.Items.Add('Left = ' + IntToStr(FSomeData.Left)) ;
    ListBox1.Items.Add('Width = ' + IntToStr(FSomeData.Width)) ;
    ListBox1.Items.Add('Height = ' + IntToStr(FSomeData.Height)) ;
    ListBox1.Items.Add('Caption = ' + FSomeData.Caption) ;
    ListBox1.Refresh ;


    { Using registry data restore the forms position }
    Top := FSomeData.Top ;
    Left := FSomeData.Left ;
    Caption := FSomeData.Caption ;

    { Close the current key }
    Reg.CloseKey ;

    { Remove it since we are done with it }    
    Reg.DeleteKey(BaseKey) ;
  finally
    Reg.CloseKey ;
    Reg.Free ;
  end ;
end;

{--------------------------------------------------------
  This is unrelated to using the registry, it is here
  more for fun.
 -------------------------------------------------------}
procedure TfrmMain.mnuReadDelphisCodeInsightClick(Sender: TObject);
var
  f:TfrmInsight ;
begin
  f := TfrmInsight.Create(Self);
  try
    CenterForm(F);
    f.ShowModal ;
  finally
    f.free ;
  end ;
end;

{--------------------------------------------------------
  Shows how to get specific information from the registry
  concerning Delphi projects.
 -------------------------------------------------------}
procedure TfrmMain.mnuGetDelphisessionsClick(Sender: TObject);
var
  aList:TStringList ;
begin
  if not ListBox1.Visible then
    ListBox1.Visible := True ;
    
  { DelphiProjects uses TRegIniFile which is is derived from TRegistry
    so it inherits all of TRegistry's properties and methods. }
  aList := TStringList.Create ;
  try
    aList.Assign(DelphiProjects) ;
    ListBox1.Clear ;
    if aList.Count > -1 then
    begin
      ListBox1.Items.Add('List of Delphi projects') ;
      ListBox1.Items.AddStrings(aList) ;
      ListBox1.ItemIndex := 0 ;
    end else ErrorMsg('Failed to get projects') ;
  finally
    aList.Free ;
  end ;
end;

{--------------------------------------------------------
  Invokes a Windows About box for the demo.
 -------------------------------------------------------}
procedure TfrmMain.mnuAboutClick(Sender: TObject);
begin
  ShellAbout(Self.Handle,PChar('Project JEDI lesson'),
    'JEDI: How to use the registry'#13#10'By Kevin S. Gallagher',
    Application.Icon.Handle) ;
end;

{--------------------------------------------------------
  Shows how to get version information for Microsoft
  Word from the registry.
 -------------------------------------------------------}
procedure TfrmMain.mnuMSWordinformationClick(Sender: TObject);
var
  cVersion:String ;
  cLocation:String ;
begin
  { Note the next line of code makes a direct reference to the
    function we are using. This is not needed, just wanted to
    show that this can be done. Sometimes in some situations
    you might need to do this, but not here. }
  cLocation := ms_word_info.GetMicrosoftWordLocation(cVersion) ;

  if length(cVersion) = 0 then    cVersion := 'Unknown' ;
  if Length(cLocation) = 0 then   cLocation := 'Unknown' ;

  if not ListBox1.Visible then    ListBox1.Visible := True ;

  with ListBox1 do
  begin
    Items.Add('Microsoft Word information from the registry') ;
    Items.Add('Location=' + cLocation ) ;
    Items.Add('Version=' + cVersion ) ;
    if Items.Count > -1 then ItemIndex := 0 ;
  end ;

end;

{--------------------------------------------------------
  Shows how to alter some specific information if Novell
  is installed. May not work on older versions for Novell
  or if the Admin has settings which may override this
  code. Also it depends on were you are in the Novell tree.
 -------------------------------------------------------}
procedure TfrmMain.ChangeNovelldefaultuser1Click(Sender: TObject);
var
  f:TfrmNovell ;
begin
  { The Outer TRY statement works along with TfrmNovell's WMCreate
    to determine if the required registry keys exist. Refer to
    TfrmNovell.WMCreate for more comments.

    The code within the Inner TRY statement is only executed if the
    registry keys are located in TfrmNovell.WMCreate event. }
  try
    f:= TfrmNovell.Create(Self) ;
    try
      CenterForm(f) ;
      f.ShowModal ;
    finally
      f.Free ;
    end ;
  except
    { cNovellError string is set in TfrmNovell.WMCreate if there
      are trappable errors i.e. registry keys not located. }
    ErrorMsg(cNovellError);
  end ;

end;

procedure TfrmMain.mnuRunonWindowsstartupClick(Sender: TObject);
var
  f: TfrmRunApps ;
begin
  f := TfrmRunApps.Create(Self) ;
  try
    CenterForm(f) ;
    f.ShowModal ;
  finally
    f.Free ;
  end ;
end;

procedure TfrmMain.mnuBrowsebranchoftheregistryClick(Sender: TObject);
var
  f:TEnumuateRegBranch ;
begin
  { NOTE: The code which runs in this form is not called OnCreate
    since it takes sometime to execute. }
  f := TEnumuateRegBranch.Create(Self) ;
  try
    CenterForm(F) ;
    f.ShowModal ;
  except
    ErrorMsg('Encountered problems') ;
  end ;
end;

procedure TfrmMain.mnuRestoreTreeViewClick(Sender: TObject);
var
  f: TfrmTreeNodeRestore ;
begin
  f := TfrmTreeNodeRestore.Create(Self) ;
  try
    CenterForm(f) ;
    f.ActiveControl := f.TreeView1 ;
    f.ShowModal ;
  except
    ErrorMsg('Encountered problems') ;
  end ;
end;

end.

