unit Novell_LogOn;

{***************************************************************************}
{ Description                                                               }
{   Where I work the users boot up thier computers, are presented with a    }
{   Novell log-in dialog, thier username is already there, they simply      }
{   put in thier password and press the Okay button. In the past someone    }
{   else would sign in, then the owner of the computer would come back and  }
{   login. The login would fail since they do not realize the default login }
{   name has changed. They _never_ consider looking at the default login    }
{   username. 99% of the time the person causing the problem was a user     }
{   support person, they then began going into the registry, search for a   }
{   key, thumb down a few keys a change the username back to what it was    }
{   prior to thier login.                                                   }
{                                                                           }
{   This utility simplifies the process, it is located in a public NOVELL   }
{   directory. When they login they first highlight the default username,   }
{   press CTRL+C, change it, login. Next pressing "WinKey" + R, type in the }
{   utility name, press CTRL+V to paste in the name. Pressing ENTER saves   }
{   the name back. This method is safer since the tech does not have to     }
{   access the registry raw.                                                }
{                                                                           }
{ Changes for use in JEDI demo                                              }
{   + Removed OS checks i.e. are we running NT...                           }
{   + Removed some in house components.                                     }
{   + Added the procedure WMCreate for demoing.                             }
{                                                                           }
{ Operating System(s)                                                       }
{   Has only been tested under Windows95                                    }
{                                                                           }
{ Version                                                                   }
{   1.00                                                                    }
{                                                                           }
{ Author                                                                    }
{   Kevin S. Gallagher                                                      }
{                                                                           }
{ Additional software                                                       }
{   Nothing                                                                 }
{                                                                           }
{ Programming language                                                      }
{   Inprise Delphi 3 or later                                               }
{                                                                           }
{ Changes                                                                   }
{   Date        Name    Description                                         }
{   02.29.00    KSG     Initial version                                     }
{***************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Registry ;

type
  TfrmNovell = class(TForm)
    txtUserName: TEdit;
    Label1: TLabel;
    cmdSave: TButton;
    cmdCancel: TButton;
    procedure cmdCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdSaveClick(Sender: TObject);
    procedure txtUserNameKeyPress(Sender: TObject; var Key: Char);
  private
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
  public
  end;

const
  GetInstance = 'SOFTWARE\Novell\Location Profiles\Profiles\Default\Instances' ;
  GetService  = 'SOFTWARE\Novell\Location Profiles\Services' ;
  GeneralError= 'Note: You may be running an unsupported version of Novell' ;
var
  frmNovell: TfrmNovell;

implementation

{$R *.DFM}


uses ShellApi, JEDI_Dialogs, MainForm ;


{--------------------------------------------------------
  The code below checks to see if the two keys exist,
  if they do then the form will be created and shown.
  If the keys are not located then the creation of this
  form is aborted. Refer to how I create the form in the
  main form and how I use a variable in the main form to
  send information back to the main form which in turn is
  displayed to the user what went wrong and possibly why.
 -------------------------------------------------------}
procedure TfrmNovell.WMCreate(var Message: TWMCreate);
var
  Reg: TRegistry ;
  bOk:Boolean ;
begin
  inherited ;

  bOk := False ;

  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE ;
      if (KeyExists(GetInstance)) and (KeyExists(GetService)) then
        bOk := True
      else
        frmMain.cNovellError :=
        'Failed to locate the following registry keys~' +
        GetInstance + '~' + GetService  + '~' +
        'This part of the demo may have failed if not using a certain ' +
        'version of Novell, or an untested version of Windows i.e. NT';
    end ;
  finally
    Reg.Free ;
  end ;

  if bOk then
    Message.Result := 0
  else
    Message.Result := -1 ;
end ;

{---------------------------------------------------------------------
  The button closes the utility without any actions taken.
 --------------------------------------------------------------------}
procedure TfrmNovell.cmdCancelClick(Sender: TObject);
begin
  Close ;
end;

{---------------------------------------------------------------------
  Opens the Windows registry if Windows95/98 is detected to get the
  entry containing the default name to logon to Novell. If Windows NT
  or Windows2000 is detected then we abort.
 --------------------------------------------------------------------}
procedure TfrmNovell.FormCreate(Sender: TObject);
var
  Reg: TRegistry ;
  aList: TStringList ;
  cServiceKey: String ;
begin
  Self.Caption := '' ;

  Reg := TRegistry.Create ;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE ;

      if OpenKey(GetInstance,False) then
      begin
        aList := TStringList.Create ;
        try
          Reg.GetValueNames(aList) ;
          if aList.Count > -1 then
          begin
            cServiceKey := aList.Strings[0] ;
            CloseKey ;
            if OpenKey(GetService + '\' + cServiceKey + '\Default',False) then
            begin
              if ValueExists('UserName') then
                txtUserName.Text := ReadString('UserName') ;
            end ;
          end else
          begin
            cmdSave.Enabled := False ;
            ErrorMsg('Failed to find reference to Service key~' +
              GeneralError) ;
          end ;
        finally
          aList.Free ;
        end ;
      end else
      begin
        cmdSave.Enabled := False ;
        ErrorMsg('failed to open HKEY_LOCAL_MACHINE\' + GetInstance + '~' +
          GeneralError) ;
      end ;
    end ;
  finally
    Reg.CloseKey ;
    Reg.Free;
  end ;
end;

{---------------------------------------------------------------------
  Used to save the text entered as the default username for the next
  login to Novell. It is not possible to validate if the text entered
  is valid. In other words, watch you spelling.

  NOTE: TRegistry.LazyWrite by default is False, this will not do for
  this operation so it has been set to True.
 --------------------------------------------------------------------}
procedure TfrmNovell.cmdSaveClick(Sender: TObject);
var
  Reg: TRegistry ;
  aList: TStringList;
  cServiceKey: String ;
  cCheck: String ;
begin

  Reg := TRegistry.Create ;
  Reg.LazyWrite := False ;

  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE ;
      if (KeyExists(GetInstance)) and (KeyExists(GetService)) then
      begin
        if OpenKey(GetInstance,False) then
        begin
          aList := TStringList.Create ;
          try
            Reg.GetValueNames(aList) ;
            if aList.Count > -1 then
            begin
              cServiceKey := aList.Strings[0] ;
              CloseKey ;
              if OpenKey(GetService + '\' + cServiceKey + '\Default',False) then
              begin
                if ValueExists('UserName') then
                begin
                  if Length(txtUserName.Text) >0 then
                  begin
                    WriteString('UserName',txtUserName.text) ;
                    cCheck := ReadString('UserName') ;
                    if CompareStr(txtUserName.text,cCheck) <> 0 Then
                      ShowMessage('Failed to change Logon name') ;
                    CloseKey ;
                    Close ;
                  end ;
                end ;
              end ;
            end else
            begin
              cmdSave.Enabled := False ;
              ShowMessage('Failed to find reference to Service key') ;
            end ;
          finally
            aList.Free ;
          end ;
        end else
        begin
          cmdSave.Enabled := False ;
          ShowMessage('failed to open HKEY_LOCAL_MACHINE\' + GetInstance) ;
        end ;
      end else
      begin
        cmdSave.Enabled := False ;
        txtUserName.Visible := False ;
        Height := Height - 100 ;
        ErrorMsg('failed to locate Novell registry keys listed below~' +
          GetService + '~' + GetInstance ) ;
      end ;
    end ;
  finally
    Reg.CloseKey ;
    Reg.Free;
  end ;
end;

{---------------------------------------------------------------------
  To assist the user, if they press ENTER in the textbox this invokes
  the save method (only if there is text in the textbox). If there is
  no text, no message is shown to inform them of the fact there is no
  text.
 --------------------------------------------------------------------}
procedure TfrmNovell.txtUserNameKeyPress(Sender: TObject; var Key: Char);
begin
  if Ord(Key) = 13 then
  begin
    if Length(Trim(txtUserName.Text)) >0 then
    begin
      Key := '0' ;
      cmdSaveClick(Sender) ;
    end ;
  end ;
end;

{---------------------------------------------------------------------
  Displays an "About" dialog when pressing the "I" button in the forms
  caption next to the close button.
 --------------------------------------------------------------------}
end.
