unit ShowCurrentUserColors;
{***************************************************************************}
{ Author                                                                    }
{   Kevin S. Gallagher                                                      }
{   gallaghe@teleport.com                                                   }
{ Description                                                               }
{   This unit shows the following                                           }
{   How to access (read) information from the registry and convert the      }
{   information to something everyone can relate to with Delphi.            }
{                                                                           }
{ Comments                                                                  }
{   For novice programmers, there is plenty of code other then accessing    }
{   the registry which you may find usefull. For instance, how to create    }
{   visual components "on the fly".                                         }
{***************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Registry;

type
  TfrmShowUserColors = class(TForm)
    Panel1: TPanel;
    StatusBar1: TStatusBar;
    cmdClose: TButton;
    ScrollBox1: TScrollBox;
    procedure FormCreate(Sender: TObject);
    procedure cmdCloseClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmShowUserColors: TfrmShowUserColors;

implementation

{$R *.DFM}

procedure TfrmShowUserColors.FormCreate(Sender: TObject);
  procedure ShowColors(cCaption, Value: String; TheTop: Integer) ;
  var
    L: TLabel ;
    P: TPanel ;
    x: Integer ;
    R: Integer ;
    G: Integer ;
    B: Integer ;
    S: String ;
  begin

    S := Value ;

    { Create labels for each system color }
    l := TLabel.Create(Self);
    l.Name := 'Label' + IntToStr(TheTop) ;
    l.Caption := cCaption ;
    l.Parent := ScrollBox1 ;
    l.Top := TheTop ;
    l.Left := 10 ;
    l.Visible := True ;

    { Create a panel to show the color for corresponding
      label above }
    P := TPanel.Create(Self) ;
    P.Name := 'Panel' + IntToStr(TheTop) ;
    P.Visible := False ;
    P.Caption := '' ;
    P.Parent := ScrollBox1 ;

    { Convert string RGB to individual integers for
      showing color below }
    x := Pos(#32,S) ;
    R := StrToInt(Copy(S,1,x -1)) ;
    Delete(S,1, x) ;

    x := Pos(#32,S) ;
    G := StrToInt(Copy(S,1,x -1)) ;
    Delete(S,1, x ) ;

    B := StrToInt(S) ;

    P.Color := TColor(Windows.rgb(R,G,B)) ;

    P.Top := L.Top + 1 ;
    P.Left := 150 ;
    P.Width := 11 ;
    P.Height := 11 ;
    P.Visible := True ;
  end ;
const
  { the key to read from in the registry }
  BaseKey = 'Control Panel\Colors' ;
var
  Reg: TRegistry ;
  aList: TStringList ;
  iCount: Integer ;
  iValue: Integer ;
  cValue: String ;
  iTop: Integer ;
begin

  Reg := TRegistry.Create ;

  try
    with Reg do
    begin
      RootKey := HKEY_CURRENT_USER ;
      if OpenKey(BaseKey, False) then
      begin
        aList := TStringList.Create ;

        { Used to stop each component from painting as they are
          created. When LockWindowUpdate is called again, every
          thing is shown at once. }
        lockWindowUpdate(handle) ;
        try
          { Fill the string list with all "Values" in the open
            key. }
          GetValueNames(aList) ;
          iCount := aList.Count ;
          CloseKey ;
          iTop := 5 ;

          { Big assumtion, that the key and values exist!!!
            I'd wonder if they were not located for the user sake. }
          for iValue := 0 to iCount -1 do
          begin
            if OpenKey(BaseKey,False) then
            begin
              { We could have passed ReadString below directly
                into ShowColors, but this is clearer. }
              cValue := ReadString(aList.Strings[iValue]) ;
              CloseKey ;
              ShowColors(aList.Strings[iValue],cValue,iTop) ;
              { label height plus fill between next label }
              Inc(iTop, 16) ;
            end ;
          end ;
        finally
          aList.Free ;
          lockWindowUpdate(0) ;
        end ;
      end ;
    end ;
  finally
    Reg.Free ;
  end ;
end;

procedure TfrmShowUserColors.cmdCloseClick(Sender: TObject);
begin
  Close ;
end;

end.
