unit Unit2;

{----------------------------------------------------------
  Simple example of how to search a branch of the registry

  Author: Peter Belows of TeamB, one of the great people
          in the Delphi world.


  KSG tweaked the code a bit.
 ---------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TEnumuateRegBranch = class(TForm)
    Memo1: TMemo;
    cmdStart: TButton;
    StatusBar1: TStatusBar;
    cmdClose: TButton;
    procedure cmdStartClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmdCloseClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    iRanIt:boolean ;
  public
    { Public declarations }
  end;

var
  EnumuateRegBranch: TEnumuateRegBranch;

implementation

{$R *.DFM}

uses registry ;

procedure TEnumuateRegBranch.cmdStartClick(Sender: TObject);
var
  indent: Integer;
  iMatches: Integer ;
  procedure EnumAllKeys( hkey: THandle );
  var
    l: TStringList;
    n: Integer;
  begin
   Inc(indent, 2);
   with Tregistry.Create do
   try
     RootKey := hkey;
     OpenKey( EmptyStr, false );
     l := TStringLIst.Create;
     try
       GetKeynames(l);
       CloseKey;
       for n := 0 To l.Count-1 do begin
         memo1.lines.add( StringOfChar(' ',indent) +l [n]);
         try
          if CompareStr('CurrentVersion',L.Strings[n]) = 0 then
            Inc(iMatches) ;
         except
         end ;
         if OpenKey( l[n], false ) then begin
           EnumAllKeys( CurrentKey );
           CloseKey;
         end;
       end;
     finally
       l.Free
     end;
   finally
     Free;
   end;
   Dec(indent, 2);
  end;
begin
  iMatches := 0 ;
  
  { Searches Key and Values for "CurrentVersion" while traversing. }
  Memo1.Clear;
  memo1.lines.add('Keys under HKEY_CURRENT_USER');
  indent := 0;
  cmdStart.Enabled := False ;
  cmdClose.Enabled := False ;
  try
    cursor := crHourGlass ;
    Memo1.Lines.BeginUpdate ;
    EnumAllKeys( HKEY_CURRENT_USER );
    iRanIt := True ;
  finally
    Memo1.Lines.EndUpdate ;
    Cursor := crDefault ;
    cmdStart.Enabled := True ;
    cmdClose.Enabled := True ;
  end ;
  
  if iMatches >0 then
    ShowMessage('Found ' + IntToStr(iMatches) + ' of "CurrentVersion"')
    else
      ShowMessage('Did not find CurrentVersion') ;
end;

procedure TEnumuateRegBranch.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  { If there a lot of entries then it will take time for the
    Memo to clean up }

  Action := caFree ;
end;

procedure TEnumuateRegBranch.cmdCloseClick(Sender: TObject);
begin
  Close ;
end;

procedure TEnumuateRegBranch.FormResize(Sender: TObject);
var
  iTop:Integer ;
begin
  cmdClose.Left := (ClientWidth - cmdClose.Width) - 2;
  cmdStart.Left := (cmdClose.Left - cmdClose.Width) - 2 ;

  Memo1.Width := ClientWidth - 4 ;

  iTop := (ClientHeight - StatusBar1.Height) - cmdClose.Height - 2 ;
  cmdStart.Top := iTop ;
  cmdClose.Top := iTop ;

  Memo1.Height := iTop - 15 ;
end;

procedure TEnumuateRegBranch.FormCreate(Sender: TObject);
var
  hMenuHandle : HMENU;
  iPos:Integer;
begin

  hMenuHandle := GetSystemMenu(Handle, FALSE);
  if (hMenuHandle <> 0) then
  begin
    DeleteMenu(hMenuHandle, SC_CLOSE, MF_BYCOMMAND);
    iPos := GetMenuItemCount(hMenuHandle);
    Dec(iPos);
    if iPos > -1 then
      DeleteMenu(hMenuHandle,iPos,MF_BYPOSITION);
  end;

end;

end.
